import React from 'react';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

var selectValues = function (selection, data, props) {
    if (selection === void 0) { selection = []; }
    if (data === void 0) { data = {}; }
    if (props === void 0) { props = {}; }
    if (selection instanceof Array) {
        return selectFromArray(selection, data);
    }
    if (typeof selection === 'object') {
        return selectFromObject(selection, data, props);
    }
    if (typeof selection === 'function') {
        return selection(data, props);
    }
    throw 'Selector should be plain object, array or function';
};
function selectFromArray(selection, data) {
    if (data === void 0) { data = {}; }
    var selectionMap = {};
    selection.forEach(function (propName) {
        var _a = propName.split(':'), key = _a[0], value = _a[1];
        if (value) {
            selectionMap[key] = get(data, value);
        }
        else {
            selectionMap[key] = data[key];
        }
    });
    return selectionMap;
}
function selectFromObject(selection, data, props) {
    var selectionMap = {};
    Object.entries(selection).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        if (typeof value === 'string') {
            selectionMap[key] = get(data, value);
        }
        else {
            selectionMap[key] = value(data, props);
        }
    });
    return selectionMap;
}
function get(obj, path, def) {
    var fullPath = path
        .replace(/\[/g, '.')
        .replace(/]/g, '')
        .split('.')
        .filter(Boolean);
    return fullPath.every(everyFunc) ? obj : def;
    function everyFunc(step) {
        return !(step && (obj = obj[step]) === undefined);
    }
}

function connectContext(Context, Component, options) {
    if (options === void 0) { options = {}; }
    return React.memo(function (props) {
        var _a = getContextSelection(Context, options, props), selectedState = _a[0], selectedActions = _a[1];
        var mergedProps = getMergedProps(__assign({}, selectedState, selectedActions), props, options.computedSelectors);
        return React.useMemo(function () { return (React.createElement(Component, __assign({}, mergedProps))); }, Object.values(selectedState));
    });
}
function connectContextFactory(Context) {
    return function (Component, options) {
        if (options === void 0) { options = {}; }
        return (connectContext(Context, Component, options));
    };
}
function useConnectedContextFactory(Context) {
    return function (options) {
        if (options === void 0) { options = {}; }
        var _a = getContextSelection(Context, options), selectedState = _a[0], selectedActions = _a[1];
        var mergedProps = getMergedProps(__assign({}, selectedState, selectedActions), {}, options.computedSelectors);
        return mergedProps;
    };
}
function getContextSelection(Context, options, props) {
    if (options === void 0) { options = {}; }
    if (props === void 0) { props = {}; }
    var context = React.useContext(Context);
    var _a = normalizedContextOptions(options), stateSelectors = _a.stateSelectors, actionSelectors = _a.actionSelectors;
    var selectedState = selectValues(stateSelectors, context.state, props);
    var selectedActions = selectValues(actionSelectors, context.actions, props);
    return [selectedState, selectedActions];
}
function mergedConnectContextFactory(contexts) {
    return function (Component, options) {
        if (options === void 0) { options = {}; }
        return React.memo(function (props) {
            var mergedState = {};
            var mergedActions = {};
            contexts.forEach(function (Context) {
                var context = React.useContext(Context);
                Object.assign(mergedState, context.state);
                Object.assign(mergedActions, context.actions);
            });
            var selection = __assign({}, selectValues(options.stateSelectors, mergedState, props), selectValues(options.actionSelectors, mergedActions, props));
            var mergedProps = getMergedProps(selection, props, options.computedSelectors);
            // DEPRECATED
            if (options.afterMerge) {
                var afterMergeProps = Object.assign({}, mergedProps, options.afterMerge(mergedProps) || {});
                return React.createElement(Component, __assign({}, afterMergeProps));
            }
            return React.createElement(Component, __assign({}, mergedProps));
        });
    };
}
function getMergedProps(selection, props, computedSelectors) {
    if (computedSelectors === void 0) { computedSelectors = {}; }
    var mergedProps = __assign({}, selection, props);
    Object.entries(computedSelectors).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        var fn = value[0], deps = value[1];
        var depValues = deps.map(function (dep) { return mergedProps[dep]; });
        var memoValues = depValues.filter(function (depValue) { return typeof (depValue) !== 'function'; });
        mergedProps[key] = React.useMemo(function () { return fn.apply(void 0, depValues); }, memoValues);
    });
    return mergedProps;
}
function normalizedContextOptions(options) {
    var normalized = {};
    // DEPRECATED
    if (options.stateMappers) {
        normalized.stateSelectors = options.stateMappers;
    }
    if (options.actionMappers) {
        normalized.actionSelectors = options.actionMappers;
    }
    return Object.assign({}, options, normalized);
}

function createContextProvider(reducer, initialState, actionCreators) {
    var Context = React.createContext({});
    return [
        function (_a) {
            var onInit = _a.onInit, props = __rest(_a, ["onInit"]);
            var _b = React.useReducer(reducer, initialState), state = _b[0], dispatch = _b[1];
            var actions = getBindedActions(actionCreators, dispatch, state);
            var contextValue = {
                state: Object.freeze(state),
                actions: Object.freeze(actions),
            };
            if (onInit) {
                React.useMemo(function () {
                    onInit(contextValue);
                }, onInit);
            }
            return (React.createElement(Context.Provider, __assign({ value: contextValue }, props)));
        },
        Context,
    ];
}
function getBindedActions(actions, dispatch, state) {
    var bindedActions = {};
    Object.entries(actions).forEach(function (_a) {
        var key = _a[0], value = _a[1];
        bindedActions[key] = value(dispatch, state);
    });
    return bindedActions;
}

export default createContextProvider;
export { connectContext, connectContextFactory, useConnectedContextFactory, mergedConnectContextFactory };
//# sourceMappingURL=index.es.js.map
